function BIDS = PlotCoReg(BIDS)
%
%
%

output_dir = fullfile(BIDS.pth,'derivatives',BIDS.config.env.derivatives_dir);
setenv('SUBJECTS_DIR',fullfile(BIDS.pth,'derivatives','freesurfer'));


if BIDS.config.env.nproc > 1
    parpool('local',BIDS.config.env.nproc)
    parfor idx = 1:numel(BIDS.subjects)
        subj = BIDS.subjects(idx).name;
        ses = BIDS.subjects(idx).session;

        logs_dir = fullfile(output_dir, 'logs', subj);

        output_mean = fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_desc-mc_mean.nii.gz']);

        tmp_pet = MRIread(output_mean);

        output_lta = fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.lta']);

        views = {'axial','coronal','sagittal'};

        coreg_qc = bids.util.jsondecode(fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.json']));

        coreg_qc.QC = {};

        for k = 1:numel(views)

            unix(['freeview -transform-volume -viewport cor' ...
                ' -v ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/mri/brainmask.mgz') ...
                ':visible=0' ...
                ':name=brainmask.mgz' ...
                ':colormap=grayscale:colorscale=0,300 ' ...
                output_mean ...
                ':name=' [subj '_' ses '_desc-mc_mean.nii.gz'] ...
                ':reg=' output_lta ...
                ':colormap=pet:colorscale=' num2str(prctile(tmp_pet.vol(:),25)) ',' num2str(prctile(tmp_pet.vol(:),100)) ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/lh.white') ...
                ':edgecolor=black' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/rh.white') ...
                ':edgecolor=black' ...
                ' -viewport ' views{k} ...
                ' -ss ' fullfile(logs_dir,[subj '_' ses '_' views{k} '_coregPET.png']) ...
                ' -quit']);

            coreg_qc.QC{end+1} = fullfile(erase(logs_dir,pwd),[subj '_' ses '_' views{k} '_coregPET.png']);

            unix(['freeview -transform-volume -viewport cor' ...
                ' -v ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/mri/brainmask.mgz') ...
                ':name=brainmask.mgz' ...
                ':colormap=grayscale:colorscale=10,150 ' ...
                output_mean ...
                ':visible=0' ...
                ':name=' [subj '_' ses '_desc-mc_mean.nii.gz'] ...
                ':reg=' output_lta ...
                ':colormap=pet:colorscale=0,3000 ' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/lh.white') ...
                ':edgecolor=black' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/rh.white') ...
                ':edgecolor=black' ...
                ' -viewport ' views{k} ...
                ' -ss ' fullfile(logs_dir,[subj '_' ses '_' views{k} '_coregMR.png']) ...
                ' -quit']);

            coreg_qc.QC{end+1} = fullfile(erase(logs_dir,pwd),[subj '_' ses '_' views{k} '_coregMR.png']);

        end

        bids.util.jsonwrite(fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.json']),coreg_qc);

    end
    delete(gcp('nocreate'));
else
    for idx = 1:numel(BIDS.subjects)
        subj = BIDS.subjects(idx).name;
        ses = BIDS.subjects(idx).session;

        logs_dir = fullfile(output_dir, 'logs', subj);

        output_mean = fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_desc-mc_mean.nii.gz']);

        tmp_pet = MRIread(output_mean);

        output_lta = fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.lta']);

        views = {'axial','coronal','sagittal'};

        coreg_qc = bids.util.jsondecode(fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.json']));

        coreg_qc.QC = {};

        for k = 1:numel(views)

            unix(['freeview -transform-volume -viewport cor' ...
                ' -v ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/mri/brainmask.mgz') ...
                ':visible=0' ...
                ':name=brainmask.mgz' ...
                ':colormap=grayscale ' ...
                output_mean ...
                ':name=' [subj '_' ses '_desc-mc_mean.nii.gz'] ...
                ':reg=' output_lta ...
                ':colormap=pet:colorscale=' num2str(prctile(tmp_pet.vol(:),25)) ',' num2str(prctile(tmp_pet.vol(:),100)) ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/lh.white') ...
                ':edgecolor=black' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/rh.white') ...
                ':edgecolor=black' ...
                ' -viewport ' views{k} ...
                ' -ss ' fullfile(logs_dir,[subj '_' ses '_' views{k} '_coregPET.png']) ...
                ' -quit']);

            coreg_qc.QC{end+1} = fullfile(erase(logs_dir,pwd),[subj '_' ses '_' views{k} '_coregPET.png']);

            unix(['freeview -transform-volume -viewport cor' ...
                ' -v ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/mri/brainmask.mgz') ...
                ':name=brainmask.mgz' ...
                ':colormap=grayscale ' ...
                output_mean ...
                ':visible=0' ...
                ':name=' [subj '_' ses '_desc-mc_mean.nii.gz'] ...
                ':reg=' output_lta ...
                ':colormap=pet:colorscale=0,3000 ' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/lh.white') ...
                ':edgecolor=black' ...
                ' --surface ' fullfile(BIDS.pth,'derivatives','freesurfer',subj,'/surf/rh.white') ...
                ':edgecolor=black' ...
                ' -viewport ' views{k} ...
                ' -ss ' fullfile(logs_dir,[subj '_' ses '_' views{k} '_coregMR.png']) ...
                ' -quit']);

            coreg_qc.QC{end+1} = fullfile(erase(logs_dir,pwd),[subj '_' ses '_' views{k} '_coregMR.png']);

        end

        bids.util.jsonwrite(fullfile(output_dir, subj, ses, ...
            'pet', [subj '_' ses '_from-pet_to-T1w_reg.json']),coreg_qc);
    end
end